//
// Name: FT30402.PRG
// Project: Fischer Technik interfacing
// Description: ft com 30402 interface driver
// Language: xBase++
// Libs : based on umFish30.dll (via ft30402.prg,obj) 
//         and serial com lib xbTools III
// Author: Rene Nooteboom
// Email: r.nooteboom2@chello.nl
// Date created: 10/05/2006 10:29
// Date updated: 02/09/2006 22:54
// Remark : maximum 4 com ports can be opened simulteanously
//
// Function FtCAD(iPortHandle,nAnalogInput) // EX=1,EY=2
// Function FtCADDirect(iPortHandle,nAnalogInput) // EX=1,EY=2
// Function FtCAnalogScan(iPortHandle)
// Function FtCExtension(iPortHandle)
// Function FtCGetCounter(iPortHandle,nE1toE16)
// Function FtCGetModeStatus(iPortHandle,nMotorNr)
// Function FtCGetMotors(iPortHandle)
// Function FtCInit(nComPort,lAskValues,nPollInterval,lWithSlave,lAnalogScan)  // all optional
// Function FtCInput(iPortHandle)
// Function FtCMotor(iPortHandle,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
// Function FtCMotorLeft(iPortHandle,nMotorNr,nSpeed)
// Function FtCMotorOff(iPortHandle,nMotorNr)
// Function FtCMotorOn(iPortHandle,nMotorNr,nSpeed)
// Function FtCMotorRight(iPortHandle,nMotorNr,nSpeed)
// Function FtCMotorSpeed(iPortHandle,nMotorNr,nDirection,nSpeed) // MON,MCW,MCCW,MOFF
// Function FtCOffNr(iPortHandle,nO1toO16)
// Function FtCOnNr(iPortHandle,nO1toO16)
// Function FtCOutput(iPortHandle,nOutput)
// Function FtCPollInterval(iPortHandle) // in ms
// Function FtCRobMotor(iPortHandle,nMotorNr,nDirection,nSpeed,nCount)
// Function FtCRobMotors(iPortHandle,nMotorStatus,nSpeedStatus,nModeStatus)
// FUNCTION FtCSensor(iPortHandle,nE1toE16)
// Function FtCSetLamp(iPortHandle,nLampNr,lOnOff)
// Function FtCSetMotors(iPortHandle,nMotorStatus) 
// Function FtCSetMotorsEx(iPortHandle,nMotorStatus,nSpeedStatus) 
// Function FtCSlave(iPortHandle)
// Function FtCUsedComPort(iPortHandle)
// PROCEDURE FtCButton(iPortHandle,nE1toE16,iSleepIdleTime) // wait for high
// PROCEDURE FtCButtonPushed(iPortHandle,nE1toE16,iSleepIdleTime) // wait for low
// Procedure FtCClearCounters(iPortHandle)
// PROCEDURE FtCClose(iPortHandle)
// Procedure FtCCloseAll()
// PROCEDURE FtCMotorsOff(iPortHandle)
// Procedure FtCSetCounter(iPortHandle,nE1toE16,nValue)
// Procedure FtCSetModeStatus(iPortHandle,nMotorNr,nMode) 
// Procedure FTCShowUsedIF()
//
#include "common.ch"
#include "inkey.ch"
#include "dll.ch"
#include "xbp.ch"
#include "xbtcom.ch"
#include "noot.ch"
#include "ft.ch"
#include "umfish30.ch"
#include "ft30402.ch"
//
Static nUmFish30Handle:=0,nUsedComPorts_:={},nPortHandlesUsed_:={}
//
Function FtCInit(nComPort,lAskValues,nPollInterval,lWithSlave,lAnalogScan)  // all optional
//
// returns : iPortHandle and -1 if error
//
// nPollInterval : in mS : if ==0 determined by OpenInterface()
//                  if lAnalogScan -> 16mS
//                  else lWithSlave -> 12mS else -> 10mS
//                 my default : 20mS
//
 LOCAL nAnalogScan,nSlave,GetList:={},oCrt,oParent,oOwner,lExit:=FALSE, ;
  cComPorts_:={},cComPort:="COMxx",i,cmsg,iPortHandle:=0
 Default nComPort TO 1,lAskValues TO TRUE
 Default nPollInterval to 20,lWithSlave to FALSE
 Default lAnalogScan to FALSE
 nSlave := If(lWithSlave,1,0)
 nAnalogScan := If(lAnalogScan,1,0)
 If Len(nUsedComPorts_) > 3
  ALARMTONE
  cmsg := "FtCInit():Error: Maximum 4 COM ports can be opened simultaneously"
  MsgBox(cmsg)
  Return -1
 EndIf
 If IS_ELEMENT(nUsedComPorts_,nComPort) .AND. !lAskValues
  ALARMTONE
  cmsg := "FtCInit():Error: COM" + str_zero(nComPort,2,0) + " already used"
  MsgBox(cmsg)
  Return -1
 EndIf
 IF lAskValues
  If AppType() == APPTYPE_VIO
   vstk_push()
  EndIf
  oParent := AppDesktop()
  oOwner := SetAppWindow()
  oCrt := XbpCrt():new(oParent,oOwner,{50,50},8,70,"30402-Interface(with umFish30.dll) configuration")
  oCrt:minmax := FALSE
  oCrt:sysmenu := FALSE
  oCrt:closeable := FALSE
  oCrt:fontname := CRT_FONT_NAME
  oCrt:create()
  SetAppWindow(oCrt)
  oCrt:setModalState(XBP_DISP_APPMODAL)
  SetAppFocus(oCrt)
  CLS
  Set Cursor on
  @ MaxRow()-1,1 SAY "Searching for available COM-ports..."
  For i:=1 to MAX_COM_PORT_NR 
   If (AScan(nUsedComPorts_,i)) < 1 // port not already in use ?
    If com_open(i)  // existing port ?
     com_close(i)  
     AAdd(cComPorts_,"COM"+str_zero(i,2,0))
    EndIf
   EndIf
  Next i
  @ MaxRow()-1,1 Clear to MaxRow()-1,MaxCol()
  If Len(cComPorts_) < 1
   ALARMTONE
   MsgBox("FtCInit(): Warning : no available COM-ports found")
  EndIf
  @ 2,2 SAY "ComPort:"
  @ 3,2 SAY "Polling Interval(mS,0=auto,try:16ms):"
  @ 4,2 SAY "Analog scan(1=yes,0=no):"
  @ 5,2 SAY "Slave Interface(1=yes,0=no):"
  @ 2,50 Get cComPort VALID {|oGet|GetListBox(oGet,cComPorts_)} Color "W/B,GR+/B"
  @ 3,50 GET nPollInterval PICTURE "9999" VALID nPollInterval >= 0
  @ 4,50 Get nAnalogScan PICTURE "9" VALID IS_BETW(nAnalogScan,0,1)
  @ 5,50 Get nSlave PICTURE "9" VALID IS_BETW(nSlave,0,1)  
  Read
  If LastKey() == K_ESC
   lExit := TRUE
  EndIf
  nComPort := Val(Right(AllTrim(cComPort),2))
  oCrt:setModalState(XBP_DISP_MODELESS) // clean up 
  oCrt:destroy()
  SetAppWindow(oOwner)
  SetAppFocus(oOwner)
  If AppType() == APPTYPE_VIO
   vstk_pop()
  EndIf  
 EndIf 
 If lExit
  Return -1
 EndIf
 If UmFish30Handle() == 0  // umfish30.dll not yet loaded ?
  If !LoadUmFish30()
   Return -1
  EndIf
 EndIf
 nUmFish30Handle := UmFish30Handle()
 If IS_BETW(nComPort,1,MAX_COM_PORT_NR)
  iPortHandle := DllCall(nUmFish30Handle,DLL_STDCALL,"csOpenInterface", ;
   nComPort,nAnalogScan,nSlave,nPollInterval)
 Else
  ALARMTONE
  cmsg := "FtCInit():Error: COM-port nr must be between 1 and " +  ;
   AllTrim(Str(MAX_COM_PORT_NR,2,0))
  MsgBox(cmsg)
  Return -1  	
 EndIf
 If iPortHandle == ftiError
  ALARMTONE
  MsgBox("FtCInit(): Error : csOpenInterface()")
  Return -1
 EndIf 
 AAdd(nUsedComPorts_,nComPort)
 AAdd(nPortHandlesUsed_,iPortHandle)
 FtCOutput(iPortHandle,0) // everything off
 Return iPortHandle // -1 if error
//
PROCEDURE FtCClose(iPortHandle)
 LOCAL iIndex
 FtCMotorsOff(iPortHandle)
 DllCall(nUmFish30Handle,DLL_STDCALL,"csCloseInterface",iPortHandle)
 iIndex := AScan(nPortHandlesUsed_,iPortHandle)
 If iIndex > 0 // clean up
  ADel(nPortHandlesUsed_,iIndex)
  ASHRINK(nPortHandlesUsed_)
  ADel(nUsedComPorts_,iIndex)
  ASHRINK(nUsedComPorts_)
 EndIf
 Return
// 
Procedure FtCCloseAll()
 LOCAL i
 For i:=1 to Len(nPortHandlesUsed_)
  FtCClose(nPortHandlesUsed_[i])
 Next i
 Return
//
Function FtCInput(iPortHandle)
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetInputs", ;
  iPortHandle) 
 Return nInputStatus
//
Function FtCOutput(iPortHandle,nOutput)
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotors",iPortHandle,nOutput)
 Return nInputStatus
//
Function FtCAD(iPortHandle,nAnalogInput) // EX=1,EY=2
 LOCAL nAnalogValue:=0
 --nAnalogInput
 nAnalogValue := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetAnalog", ;
  iPortHandle,nAnalogInput)
 Return nAnalogValue
//
Function FtCADDirect(iPortHandle,nAnalogInput) // EX=1,EY=2
 LOCAL nAnalogValue:=0
 --nAnaloginput
 nAnalogValue := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetAnalogDirect", ;
  iPortHandle,nAnalogInput)
 Return nAnalogValue
//
FUNCTION FtCSensor(iPortHandle,nE1toE16)
 LOCAL lSensor
 lSensor := (DllCall(nUmFish30Handle,DLL_STDCALL,"csGetInput", ;
  iPortHandle,nE1toE16) == 1)
RETURN lSensor
//
PROCEDURE FtCMotorsOff(iPortHandle)
 FtCOutput(iPortHandle,0)
RETURN
//
Procedure FtCSetCounter(iPortHandle,nE1toE16,nValue)
 DllCall(nUmFish30Handle,DLL_STDCALL,"csSetCounter",iPortHandle,nE1toE16, ;
  nValue)
Return  
//
Procedure FtCClearCounters(iPortHandle)
 DllCall(nUmFish30Handle,DLL_STDCALL,"csClearCounters",iPortHandle)
Return  
//
Function FtCGetCounter(iPortHandle,nE1toE16)
 LOCAL nCount:=0
 nCount := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetCounter",iPortHandle, ;
  nE1toE16)
Return nCount
//
PROCEDURE FtCButton(iPortHandle,nE1toE16,iSleepIdleTime) // wait for high
 Default iSleepIdleTime to 1 // 1/100s
 If iSleepIdleTime > 0
  DO WHILE !FtCSensor(iPortHandle,nE1toE16)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE !FtCSensor(iPortHandle,nE1toE16)
  EndDo	
 EndIf
RETURN
//
PROCEDURE FtCButtonPushed(iPortHandle,nE1toE16,iSleepIdleTime) // wait for low
 Default iSleepIdleTime to 1 // 1/100s
 If iSleepIdleTime > 0
  DO WHILE FtCSensor(iPortHandle,nE1toE16)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE FtCSensor(iPortHandle,nE1toE16)
  EndDo	
 EndIf
Return
//
Function FtCMotorSpeed(iPortHandle,nMotorNr,nDirection,nSpeed) // MON,MCW,MCCW,MOFF
 LOCAL nInputStatus:=0
 Default nSpeed to FULL_MOTOR_SPEED
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotorEx", ;
  iPortHandle,nMotorNr,nDirection,nSpeed)
 Return nInputStatus
//
Function FtCMotor(iPortHandle,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotor", ;
  iPortHandle,nMotorNr,nDirection)
 Return nInputStatus
//
Function FtCGetMotors(iPortHandle)
 LOCAL nOutputStatus:=0
 nOutputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetMotors", ;
  iPortHandle) 
 Return nOutputStatus
//
Function FtCGetModeStatus(iPortHandle,nMotorNr)
 LOCAL nMode:=0
 nMode := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetModeStatus", ;
  iPortHandle,nMotorNr) 
 Return nMode
//
Procedure FtCSetModeStatus(iPortHandle,nMotorNr,nMode) 
  DllCall(nUmFish30Handle,DLL_STDCALL,"csSetModeStatus", ;
  iPortHandle,nMotorNr,nMode)
 Return 
//
Function FtCSetMotors(iPortHandle,nMotorStatus) 
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotors", ;
  iPortHandle,nMotorStatus)
 Return nInputStatus
//
Function FtCSetMotorsEx(iPortHandle,nMotorStatus,nSpeedStatus) 
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotorsEx", ;
  iPortHandle,nMotorStatus,nSpeedStatus)
 Return nInputStatus
//
Function FtCMotorLeft(iPortHandle,nMotorNr,nSpeed)
 LOCAL nInputStatus:=0
 nInputStatus := FtCMotorSpeed(iPortHandle,nMotorNr,ftiLeft,nSpeed)
Return nInputStatus
//
Function FtCMotorRight(iPortHandle,nMotorNr,nSpeed)
 LOCAL nInputStatus:=0
 nInputStatus := FtCMotorSpeed(iPortHandle,nMotorNr,ftiRight,nSpeed)
Return nInputStatus
//
Function FtCMotorOff(iPortHandle,nMotorNr)
 LOCAL nInputStatus:=0
 nInputStatus := FtCMotor(iPortHandle,nMotorNr,ftiOff)
Return nInputStatus
//
Function FtCMotorOn(iPortHandle,nMotorNr,nSpeed)
 LOCAL nInputStatus:=0
 nInputStatus := FtCMotorSpeed(iPortHandle,nMotorNr,ftiOn,nSpeed)
Return nInputStatus
//
Function FtCAnalogScan(iPortHandle)
 LOCAL lAnalogScan:=FALSE
 lAnalogScan := (DllCall(nUmFish30Handle,DLL_STDCALL, ;
  "csGetAnalogScan",iPortHandle) == 1)
 Return lAnalogScan
//
Function FtCSlave(iPortHandle)
 LOCAL lSlave:=FALSE
 lSlave := (DllCall(nUmFish30Handle,DLL_STDCALL,"csGetSlave",iPortHandle) == 1)
 Return lSlave
//
Function FtCExtension(iPortHandle)
 Return FtCSlave(iPortHandle)
//
Function FtCPollInterval(iPortHandle) // in ms
 LOCAL nPollInterval:=0
 nPollInterval := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetPollInterval", ;
  iPortHandle)
 Return nPollInterval  // in ms
//
Function FtCSetLamp(iPortHandle,nLampNr,lOnOff)
 LOCAL nInputStatus:=0,nOnOff
 Default lOnOff to TRUE
 nOnOff := If(lOnOff,1,0)
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetLamp", ; 
   iPortHandle,nLampNr,nOnOff)
 Return nInputStatus
//
Function FtCOnNr(iPortHandle,nO1toO16)
 Return FtCSetLamp(iPortHandle,nO1toO16,TRUE)
//
Function FtCOffNr(iPortHandle,nO1toO16)
 Return FtCSetLamp(iPortHandle,nO1toO16,FALSE)
//
//
// RobMode
// 
//  motor    end input(Ex)   impuls input counter
//    1
//    2
//    3            5                 6
//    4            7                 8
//    5            9                10
//    6           11                12
//    7           13                14
//    8           15                16
//
// note assumes ftiLinks(0x01) directed to end input !
//
// example csRobMotors(iPortHandle,0x9,0xF6,0x10)
//
//  0x9 = 1001 = M1 = ftiLeft(01)
//               M2 = ftiRight(10)
//
//  0xF6 = 1111 0110 = speed M1 = 6 (0110)
//                     speed M2 = F = 15
//
//  0x10 = 0001 0000 = M1 in normal mode (0000)
//                     M2 in robomode (0001)
//
Function FtCRobMotors(iPortHandle,nMotorStatus,nSpeedStatus,nModeStatus)
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csRobMotors", ;
  iPortHandle,nMotorStatus,nSpeedStatus,nModeStatus)
 Return nInputStatus
//
Function FtCRobMotor(iPortHandle,nMotorNr,nDirection,nSpeed,nCount)
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csRobMotor", ;
  iPortHandle,nMotorNr,nDirection,nSpeed,nCount)
 Return nInputStatus
//
Function FtCUsedComPort(iPortHandle)
 LOCAL iIndex,nComPort:=0
 iIndex := AScan(nPortHandlesUsed_,iPortHandle)
 If iIndex > 0
  nComPort := nUsedComPorts_[iIndex]
 EndIf
 Return nComPort
//
Procedure FTCShowUsedIF()
 LOCAL cmsg:="FT interface uses:",i
 FOR i := 1 TO Len(nUsedComPorts_)
  cmsg += " COM" + str_zero(nUsedComPorts_[i],2,0) + ;
   If(FtCSlave(nPortHandlesUsed_[i]),"(+Slave)","") 
 Next i
 MsgBox(cmsg)
RETURN
//
// einde Ft30402.prg
//
